Attribute VB_Name = "modException"
Option Explicit

' Angepate Version von 'CopyMemory' zum Folgen der Zeiger bei
' 'EXCEPTION_RECORD'-Ketten.
Private Declare Sub CopyExceptionRecord Lib "kernel32.dll" Alias "RtlMoveMemory" (ByRef Destination As EXCEPTION_RECORD, ByVal Source As Long, ByVal Length As Long)

Private Declare Function SetUnhandledExceptionFilter Lib "kernel32.dll" (ByVal lpTopLevelExceptionFilter As Long) As Long

' Rckgabewerte fr die Rckrufprozedur (Ausnahmebehandlungsprozedur).
Private Const EXCEPTION_CONTINUE_EXECUTION As Long = -1&
Private Const EXCEPTION_CONTINUE_SEARCH As Long = 0&
Private Const EXCEPTION_EXECUTE_HANDLER As Long = 1&

Private Const EXCEPTION_NONCONTINUABLE As Long = &H1&
Private Const EXCEPTION_CONTINUABLE As Long = &H0&  ' Nicht im Win32-API als
                                                    ' Konstante definiert.

' Anzahl der Elemente im Feld 'Information' der 'EXCEPTION_RECORD'-Struktur.
Private Const EXCEPTION_MAXIMUM_PARAMETERS As Long = 15&

' Datenstruktur zum Speichern von Informationen ber den eingetretenen Fehler.
Private Type EXCEPTION_RECORD
    ExceptionCode As Long
    ExceptionFlags As Long
    pExceptionRecord As Long    ' Zeiger auf einen anderen 'EXCEPTION_RECORD'-Eintrag.
    ExceptionAddress As Long
    NumberParameters As Long
    Information(0 To EXCEPTION_MAXIMUM_PARAMETERS - 1) As Long
End Type

' Prozessorspezifische Struktur mit Informationen zur Ausnahme. Diese ist
' nicht weiter von Bedeutung.
Private Const SIZE_OF_80387_REGISTERS As Long = 80&
Private Const MAXIMUM_SUPPORTED_EXTENSION As Long = 512&

Private Type FLOATING_SAVE_AREA
    ControlWord     As Long
    StatusWord      As Long
    TagWord         As Long
    ErrorOffset     As Long
    ErrorSelector   As Long
    DataOffset      As Long
    DataSelector    As Long
    RegisterArea(0 To SIZE_OF_80387_REGISTERS - 1) As Byte
    Cr0NpxState     As Long
End Type

Private Type CONTEXT
    ContextFlags    As Long
    iDr0            As Long
    iDr1            As Long
    iDr2            As Long
    iDr3            As Long
    iDr6            As Long
    iDr7            As Long
    FloatSave       As FLOATING_SAVE_AREA
    regGs           As Long
    regFs           As Long
    regEs           As Long
    regDs           As Long
    regEdi          As Long
    regEsi          As Long
    regEbx          As Long
    regEdx          As Long
    regEcx          As Long
    regEax          As Long
    regEbp          As Long
    regEip          As Long
    regCs           As Long
    regFlag         As Long
    regEsp          As Long
    regSs           As Long
    ExtendedRegisters(0 To MAXIMUM_SUPPORTED_EXTENSION - 1) As Byte
End Type

Private Type EXCEPTION_POINTERS
    ExceptionRecord As Long     ' Zeiger auf eine 'EXCEPTION_RECORD'-Struktur.
    ContextRecord As Long       ' Zeiger auf eine 'CONTEXT'-Struktur.
End Type

Private Declare Function SetErrorMode Lib "kernel32.dll" (ByVal wMode As Long) As Long

Private Const SEM_FAILCRITICALERRORS As Long = &H1&
Private Const SEM_NOGPFAULTERRORBOX As Long = &H2&
Private Const SEM_NOALIGNMENTFAULTEXCEPT As Long = &H4&
Private Const SEM_NOOPENFILEERRORBOX As Long = &H8000&

' Funktionszeiger auf die Fehlerbehandlungsprozedur von Windows.
Private m_lpOldExceptionProc As Long

' Hinweis:  Ausgeblendetes Mitglied.
Public g_eh As CExceptionHandler
Attribute g_eh.VB_VarMemberFlags = "40"

' Aktiviert die Fehlerbehandlung in der Anwendung.
'
' Hinweis:  Ausgeblendetes Mitglied.
Public Sub StartExceptionHandler()
Attribute StartExceptionHandler.VB_MemberFlags = "40"
    
    ' Folgender Aufruf ist unter Windows XP nicht erforderlich.
    Call SetErrorMode(SEM_NOGPFAULTERRORBOX)
    m_lpOldExceptionProc = SetUnhandledExceptionFilter(AddressOf ExceptionHandler)
End Sub

' Setzt auf die Standardbehandlung fr unbehandelte Fehler von Windows zurck.
'
' Hinweis:  Ausgeblendetes Mitglied.
Public Sub StopExceptionHandler()
Attribute StopExceptionHandler.VB_MemberFlags = "40"
    Call SetUnhandledExceptionFilter(EXCEPTION_EXECUTE_HANDLER)
    m_lpOldExceptionProc = 0&
End Sub

' Diese Funktion wird aufgerufen, wenn eine unbehandelte Ausnahme auftritt. Von
' hier aus wird an die Behandlungsklasse weitergeleitet, da eine Ausnahme
' aufgetreten ist.
'
' Hinweis:  Ausgeblendetes Mitglied.
Public Function ExceptionHandler(ByRef lpException As EXCEPTION_POINTERS) As Long
Attribute ExceptionHandler.VB_MemberFlags = "40"
    
    ' Fortsetzen der Ausfhrung bei einem Fehler.
    ExceptionHandler = EXCEPTION_CONTINUE_EXECUTION
    
    ' Der Anwendung mitteilen, da es zu einer Ausnahme gekommen ist. Wenn
    ' 'lpException.ExceptionRecord' ungleich 0 ist, handelt es sich um eine
    ' Kette von verschachtelten Ausnahmen. In diesem Fall mu den Zeigern
    ' zurck bis zur ursprnglichen Ausnahme gefolgt werden.
    Dim er As EXCEPTION_RECORD
    
    ' Ermitteln des aktuellen Ausnahmeeintrags.
    Call CopyExceptionRecord(er, lpException.ExceptionRecord, LenB(er))

    ' Folgen der Zeiger bis zur ursprnglichen Ausnahme.
    Do Until er.pExceptionRecord = 0&
        Call CopyExceptionRecord(er, er.pExceptionRecord, LenB(er))
    Loop
    
    Dim ei As CExceptionInfo
    Set ei = New CExceptionInfo
    With ei
        .Address = er.ExceptionAddress
        .Code = er.ExceptionCode
        .Continuable = (er.ExceptionFlags = EXCEPTION_CONTINUABLE)
    End With
    Call g_eh.OnExceptionOccured(ei)
End Function
